
// Dark GDK - The Game Creators - www.thegamecreators.com

// the wizard has created a very simple 3D project that uses Dark GDK
// it can be used as a starting point in making your own 3D games

// whenever using Dark GDK you must ensure you include the header file
#include "DarkGDK.h"
#include "Object3d.h"

// the main entry point for the application is this function
void DarkGDK ( void )
{
	// in this application we are going to create some 3D objects
	// and position them on screen

	// when starting a Dark GDK program it is useful to set global
	// application properties, we begin by turning the sync rate on,
	// this means we control when the screen is updated, we also set
	// the maximum rate to 60 which means the maximum frame rate will
	// be set at 60 frames per second
	dbSyncOn   ( );
	dbSyncRate ( 60 );

	// set our random seed to a value from the timer, this will help
	// to ensure each time we run our program the random values appear
	// more random
	dbRandomize ( dbTimer ( ) );

	int objCounter=0;
	int temp = 0;
	int dummy[100];

	dbMakeObjectCube(1000, 30);
	dbShowMouse();


	// This is the array of spheres that make up the big cube
	Object3d myObj[280*2];

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, i,-35,-35); // X=0:70,0,0
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, -35,i,-35); // 0,Y=0:70,0	
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, i,35,-35); // 0:70,70,0	
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, 35,i,-35); // 70,0:70,0	
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, i,-35,35); // X=0:70,0,0
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, -35,i,35); // 0,Y=0:70,0	
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, i,35,35); // 0:70,70,0	
		temp++;
	}

	for (int i=-35; i<35;i++) {
		myObj[temp].setup(temp, 35,i,35); // 70,0:70,0	
		temp++;
	}

	// move our camera back so we can view the objects
	dbPositionCamera ( 35, 35, -70 );

	// now we come to our main loop, we call LoopGDK so some internal
	// work can be carried out by the GDK
	while ( LoopGDK ( ) )
	{
		// display some text on screen
		dbText ( 0, 0, "Use the up and down arrow keys to move the camera" );

		// move the camera forwards
		if ( dbUpKey ( ) )
			dbMoveCamera ( 1 );

		// move the camera backwards
		if ( dbDownKey ( ) )
			dbMoveCamera ( -1 );

		/*if (dbLeftKey() )
			myObj->rotateLeft();

		if (dbRightKey())
			myObj->rotateRight();
		*/

		if (dbMouseClick()) {
			int objPicked = dbPickObject(dbMouseX(), dbMouseY(), 0, temp);
			if (objPicked != 0) {
				dbColorObject ( objPicked, dbRgb ( 0, 200, 0) );

				// get the object location 
				float x = dbObjectPositionX(objPicked);
				float y = dbObjectPositionY(objPicked);
				float z = dbObjectPositionZ(objPicked);
				
				// Now create a force vector from the camera to that location
				float force[3] = {0, 0, 5};

				// These two lines determine if the whole cube rotates or just one
				// of the spheres.
				//for (int i=0; i<temp; i++) myObj[i].addForce(force, x, y, z);
				myObj[objPicked].addForce(force, x, y, z);
			}
		}

		// Update physics simulation
		for (int i=0; i<temp; i++) myObj[i].updatePhysics();
		for (int i=0; i<temp; i++) myObj[i].updatePosition();
		
		// here we make a call to update the contents of the screen
		dbSync ( );
	}

	// before quitting delete our objects
	for ( int i = 1; i < temp; i++ )
		dbDeleteObject ( i );

	// and now everything is ready to return back to Windows
	return;
}