/* This class represents a 3x3 matrix.
 * In this code it is typically used as a rotation
 * or orientation matrix
 */
#include "Matrix3.h"
#include "DarkGDK.h"

// Construct an Matrix3 object from the array.
// assume indexes ook like this:
//      0 1 2
//      3 4 5
//      6 7 8 
Matrix3::Matrix3(float mat[9]) {

	this->piOver2 = acos(-1.0)/2.0f;
	this->radToDeg = 90 / piOver2; 

	for (int i=0;i<9;i++) {
		this->m[i] = mat[i];
	}
	

}

// Construct an identity matrix
Matrix3::Matrix3(void) {	
	for (int i=0;i<9;i++) {
		this->m[i] = 0;
	}
	
	this->m[0] = 1;
	this->m[4] = 1;
	this->m[8] = 1;

	this->piOver2 = acos(-1.0)/2.0f;
	this->radToDeg = 90 / piOver2; 

}

// Decide if a number is close to another
bool Matrix3::closeTo(float num, int val) {

	bool cl = (num > val - 1e-2) && (num < val + 1e-2);
	return cl;
}
void Matrix3::debugShowStr(float f) {
		char textStr [50];
		sprintf(textStr,"Ang:%f  %f",f, m[3]);
		dbText(30,80,textStr);
}

void Matrix3::set(int index, float val) {
	this->m[index] = val;
}

// Algorithm from 
// www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToEuler/
void Matrix3::getRotationAngles(float *xr, float *yr, float *zr) {


	// heading is y, theta, applied first, 
	// attitude is z, psi, (applied second),
	// bank is x, phi, applied 3rd 
	if (m[3] > 1){
		*yr = atan2(m[2],m[8])*this->radToDeg;
		*zr = this->piOver2*this->radToDeg;
		if ( dbABS(90-prevX) < 5) *xr = 90;
		else if ( dbABS(-90 - prevX) < 5) *xr = -90;
		else *xr = 0;
	} else if (m[3] < -1 ) {
		*yr =  atan2(m[2],m[8])*this->radToDeg;
		*zr = -this->piOver2*this->radToDeg;
		if ( dbABS(90-prevX) < 5) *xr = 90;
		else if ( dbABS(-90 - prevX) < 5) *xr = -90;
		else *xr = 0;

	} else { // Not close to one
		*yr = atan2(-m[6],m[0])*this->radToDeg; // heading
		*zr = asin(m[3])*this->radToDeg; // 
		//float t = m[0]/dbCOS(*yr); // This is now cos(att)
		//*zr = atan2(m[3], t)*this->radToDeg;
		*xr = atan2(-m[5],m[4])*this->radToDeg; // bank
	}

	getClose(prevX, xr, 360);
	getClose(prevY, yr, 360);
	getClose(prevZ, zr, 360);

	getClose(prevX, xr, 180);
	getClose(prevY, yr, 180);
	getClose(prevZ, zr, 180);

	// DEBUGGING
	if (dbABS(prevZ - *zr) > 8) {
		debugShowStr(*zr);
	} 
	prevZ = *zr;
	prevX = *xr;
	prevY = *yr;
	
	debugShowStr(*zr);
}

// Get a value that's a multiple of 180 (closest to previous value)
void Matrix3::getClose(float prev, float * newVal, float modifier) {
	
	float dist=1e10;
	float newDist;
	float finalVal;
	float bestVal; // Best value we've found

	for (int i=-1;i<2;i++) {
		finalVal = *newVal+(i*modifier);
		newDist = dbABS(prev-finalVal);
		if (newDist < dist) {
			bestVal = finalVal;
			dist = newDist;
		}
	}
	*newVal = bestVal;
}

/*
 // Algorithm from http://www.gregslabaugh.name/publications/euler.pdf
 void Matrix3::getRotationAngles(float *xr, float *yr, float *zr) {

	if (closeTo(m[6],1) || closeTo(m[6], -1) ) {
		*zr = 0;
		float delta = atan2(m[1],m[2]);
		if (m[6] < 0) {
			*xr = piOver2*this->radToDeg;
			*yr = delta*this->radToDeg;
		} else {
			*xr = -piOver2*this->radToDeg;
			*yr = delta*this->radToDeg;
		}
		
	} else { // Not close to one
		*xr = -dbASIN(m[6]); 
		float cosTh = dbCOS(*xr);
		*yr = atan2( m[7]/cosTh, m[8]/cosTh)*this->radToDeg;
		*zr = atan2( m[3]/cosTh, m[0]/cosTh)*this->radToDeg;
	}
}
*/

// Multiply this matrix by another and store 
// in this matrix
//  this = this * o;
void Matrix3::multiply(Matrix3 * o) {
	float temp[9];

	for (int row=0; row<3; row++) {
		for(int col=0; col<3; col++) {
			temp[row*3 + col] = 
				m[row*3]    * o->m[col]   +
				m[row*3 + 1]* o->m[col+3] +
				m[row*3 + 2]* o->m[col+6];
		}
	}

	for (int i=0; i<9;i++) {
		m[i] = temp[i];
	}
}

// Multiply this matrix by another and store 
// in dest matrix
//  dest = this * o;
void Matrix3::multiply(Matrix3 * o, Matrix3 * dest) {

	for (int row=0; row<3; row++) {
		for(int col=0; col<3; col++) {
			dest->m[row*3 + col] = 
				m[row*3]    * o->m[col]   +
				m[row*3 + 1]* o->m[col+3] +
				m[row*3 + 2]* o->m[col+6];
		}
	}
}

// Multiply a matrix (this) by a column vector v.
// Store resulting column vector in dest
void Matrix3::multiply(float v[3], float dest[3]) {

	dest[0] = m[0]*v[0] +
			  m[1]*v[1] +
			  m[2]*v[2];

	dest[1] = m[3]*v[0] +
			  m[4]*v[1] +
			  m[5]*v[2];

	dest[2] = m[6]*v[0] +
			  m[7]*v[1] +
			  m[8]*v[2];
}

void Matrix3::copy(Matrix3 * src) {

	for (int i=0; i<9; i++) m[i] = src->m[i];
}

// Take the current matrix and reorthogonalize it.
// This is done by computing the polar decomposition, then
// taking the "U" matrix
void Matrix3::reorthogonalize() {
	
	// P = sqrt(At * A)
	int rc;
	float w[3];
	float v[3][3];
	float tempM[3][3];

	for (int i=0; i<3; i++) 
		for(int j=0; j<3; j++) 
			tempM[i][j] = m[i*3 + j];

	rc = dsvd(tempM, 3,3, w, v);

	// Now the matrix we want is u*v = tempM*v
	for (int row=0; row<3; row++) {
		for (int col=0; col<3; col++) {
			m[(row*3)+col] = tempM[row][0]*v[0][col] +
							 tempM[row][1]*v[1][col] +
							 tempM[row][2]*v[2][col];
		}
	}

	// Close to zero is zero
	for (int i=0;i<9;i++) 
		if (m[i] < 0.0001 && m[i] > -0.0001) m[i] = 0;

}

void Matrix3::normalize(float v[3]) {
	float l = sqrt(v[0]*v[0] + v[1]*v[1] + v[2]*v[2]);

	if (l != 0) {
		v[0] = v[0] / l;
		v[1] = v[1] / l;
		v[2] = v[2] / l;
	}

}

void cross(float a[3], float b[3], float dest[3]) {
	
	dest[0] = (a[1]*b[2] - a[2]*b[1]);
	dest[1] = (a[2]*b[0] - a[0]*b[2]);
	dest[2] = (a[0]*b[1] - a[1]*b[0]);
}

// This code is modeled on Chris Hecker's code directly
void Matrix3::OrthonormalizeOrientation()
{
    float X[3] = {m[0], m[3],m[6]};
	float Y[3] = {m[1], m[4],m[7]};
	float Z[3] = {m[2], m[5],m[8]};
	
	// Normalize X
	normalize(X);

	float XcrossY[3];
	cross(X,Y, Z);
	normalize(Z);
	cross(Z,X, Y);
	normalize(Y);
	

	m[0] = X[0]; m[1] = Y[0]; m[2] = Z[0];  
	m[3] = X[1]; m[4] = Y[1]; m[5] = Z[1];
	m[6] = X[2]; m[7] = Y[2]; m[8] = Z[2];

}

// Add o to this matrix
void Matrix3::add(Matrix3 *o) {
	for (int i=0; i<9; i++) 
		m[i] = m[i] + o->m[i];
}
void Matrix3::transpose(Matrix3 * dest) {

	dest->set(0, this->m[0]);
	dest->set(1, this->m[3]);
	dest->set(2, this->m[6]);
	dest->set(3, this->m[1]);
	dest->set(4, this->m[4]);
	dest->set(5, this->m[7]);
	dest->set(6, this->m[2]);
	dest->set(7, this->m[5]);
	dest->set(8, this->m[8]);
}