// This class represents a 3D object.
//
// Author: Dan Fleck

#pragma once

#include "Matrix3.h"

class Object3d {

public:
	Object3d::Object3d();
	void Object3d::setup(int id, int xL, int yL, int zL);
	void Object3d::addForce(float force[3], int x, int y, int z);
	void Object3d::updatePhysics();
	void Object3d::updatePosition();

private:
	float totalForce[3]; // How much force is currently acting on the body
	float totalTorque[3]; // Total torque
	float pointLoc[3];

	int id; // Object ID
	int tempCounter; // Is force on or off?
	float th; // DEBUG
	Matrix3 * A; // Object orientation
	Matrix3 * rotLeftM; // These should be constants somewhere else
	Matrix3 * rotRightM;// These should be constants somewhere else

	float o[3]; // Orientation (Euler) angles x,y,z
	float M; // Total Mass
	float r[3]; // Position
	float v[3]; // Velocity
	Matrix3 * Iinv; // Inverse Intertia tensor from CM
	float L[3]; // Angular momentum
	Matrix3 * Icm; // Inertia in world coords
	float w[3]; // Angular velocity

	float width, height, depth;
	void Object3d::createInertiaTensor(Matrix3 * I, float a, float b, float c, float M);

	void Object3d::setOrientation();
	void Object3d::computeIcm();
	void Object3d::computeAngularVelocity();
	void Object3d::updateOrientation(Matrix3 * A, float h, float w[3]);
	void Object3d::debugShowStr() ;
	void Object3d::rotateObject(float newOrientation[3]);
};